' ----------------------------------------------------------------------------------
' Microsoft Developer & Platform Evangelism
' 
' Copyright (c) Microsoft Corporation. All rights reserved.
' 
' THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, 
' EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES 
' OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.
' ----------------------------------------------------------------------------------
' The example companies, organizations, products, domain names,
' e-mail addresses, logos, people, places, and events depicted
' herein are fictitious.  No association with any real company,
' organization, product, domain name, email address, logo, person,
' places, or events is intended or should be inferred.
' ----------------------------------------------------------------------------------

Imports BingMapControl.Client.GeocodeService
Imports Microsoft.Maps.MapControl
Imports System.ServiceModel

Partial Public Class BingMap
    Inherits UserControl

#Region "Constants"
    Private Const AddressPropertyName As String = "Address"
    Private Const ApplicationKeyPropertyName As String = "ApplicationKey"
    Private Const GeocodeUri As String = "http://dev.virtualearth.net/webservices/v1/geocodeservice/GeocodeService.svc"
#End Region
#Region "Private Fields"
    Private Shared GeocodeAddress As New EndpointAddress(BingMap.GeocodeUri)
#End Region


    Public Sub New()
        InitializeComponent()
    End Sub

#Region "Dependency Properties"

    Public Shared ReadOnly ApplicationKeyProperty As DependencyProperty = DependencyProperty.Register(BingMap.ApplicationKeyPropertyName, GetType(String), GetType(BingMap), New PropertyMetadata(AddressOf OnApplicationKeyChanged))

    Public Shared ReadOnly AddressProperty As DependencyProperty = DependencyProperty.Register(BingMap.AddressPropertyName, GetType(String), GetType(BingMap), New PropertyMetadata(AddressOf OnAddressChanged))

#End Region

#Region "CLR Properties"

    Public Property ApplicationKey() As String
        Get
            Return DirectCast(MyBase.GetValue(BingMap.ApplicationKeyProperty), String)
        End Get
        Set(ByVal value As String)
            MyBase.SetValue(BingMap.ApplicationKeyProperty, value)
        End Set
    End Property

    Public Property Address() As String
        Get
            Return DirectCast(MyBase.GetValue(BingMap.AddressProperty), String)
        End Get
        Set(ByVal value As String)
            MyBase.SetValue(BingMap.AddressProperty, value)
        End Set
    End Property

#End Region

#Region "Callbacks"

    Private Shared Sub OnApplicationKeyChanged(ByVal d As DependencyObject, ByVal e As DependencyPropertyChangedEventArgs)
        DirectCast(d, BingMap).OnApplicationKeyChanged(DirectCast(e.OldValue, String), DirectCast(e.NewValue, String))
    End Sub

    Private Sub OnApplicationKeyChanged(ByVal oldApplicationKey As String, ByVal newApplicationKey As String)
        Me.BingMapControl.CredentialsProvider = New ApplicationIdCredentialsProvider(Me.ApplicationKey)

        Me.NavigationToAddress(Me.Address)
    End Sub

    Private Shared Sub OnAddressChanged(ByVal d As DependencyObject, ByVal e As DependencyPropertyChangedEventArgs)
        DirectCast(d, BingMap).OnAddressChanged(DirectCast(e.OldValue, String), DirectCast(e.NewValue, String))
    End Sub

    Private Sub OnAddressChanged(ByVal oldAddress As String, ByVal newAddress As String)
        Me.NavigationToAddress(newAddress)
    End Sub

#End Region

#Region "Event Handlers"

    Private Sub GeocodeServiceClient_GeocodeCompleted(ByVal sender As Object, ByVal e As GeocodeCompletedEventArgs)
        Dim response As GeocodeResponse = e.Result

        ' Ensure that the response is valid
        If response IsNot Nothing Then
            ' Take the first result
            Dim result As GeocodeResult = response.Results.FirstOrDefault()

            If result IsNot Nothing Then
                Me.BingMapControl.Center = result.BestView.Center

                '"Create Address Pin"
                'Set a pushpin
                Dim addresspin As New Pushpin()
                addresspin.Location = result.BestView.Center
                Me.BingMapControl.Children.Add(addresspin)

                '"Calculate Zoom Level"

                'Get height/width of best view in map
                Dim height As [Double] = result.BestView.Height * 111000
                Dim width As [Double] = result.BestView.Width * 111000

                'Get desired zoom in m/pixel
                'Assume that the smaller axis determines zoom
                Dim desiredZoom As [Double]
                If Me.ActualHeight < Me.ActualWidth Then
                    desiredZoom = height / Me.ActualHeight
                Else
                    desiredZoom = width / Me.ActualWidth
                End If

                Dim desiredZoomLevel As Integer

                ' Map m/pixel to a Silverlight Zoom level
                If desiredZoom > 0 AndAlso desiredZoom <= 0.3 Then
                    desiredZoomLevel = 19
                ElseIf desiredZoom > 0.3 AndAlso desiredZoom <= 0.6 Then
                    desiredZoomLevel = 18
                ElseIf desiredZoom > 0.6 AndAlso desiredZoom <= 1.19 Then
                    desiredZoomLevel = 17
                ElseIf desiredZoom > 1.19 AndAlso desiredZoom <= 2.39 Then
                    desiredZoomLevel = 16
                ElseIf desiredZoom > 2.39 AndAlso desiredZoom <= 4.78 Then
                    desiredZoomLevel = 15
                ElseIf desiredZoom > 4.78 AndAlso desiredZoom <= 9.55 Then
                    desiredZoomLevel = 14
                ElseIf desiredZoom > 9.55 AndAlso desiredZoom <= 19.11 Then
                    desiredZoomLevel = 13
                ElseIf desiredZoom > 19.11 AndAlso desiredZoom <= 38.22 Then
                    desiredZoomLevel = 12
                ElseIf desiredZoom > 38.22 AndAlso desiredZoom <= 76.44 Then
                    desiredZoomLevel = 11
                ElseIf desiredZoom > 76.44 AndAlso desiredZoom <= 152.87 Then
                    desiredZoomLevel = 10
                ElseIf desiredZoom > 152.87 AndAlso desiredZoom <= 305.75 Then
                    desiredZoomLevel = 9
                ElseIf desiredZoom > 305.75 AndAlso desiredZoom <= 611.5 Then
                    desiredZoomLevel = 8
                ElseIf desiredZoom > 611.5 AndAlso desiredZoom <= 1222.99 Then
                    desiredZoomLevel = 7
                ElseIf desiredZoom > 1222.99 AndAlso desiredZoom <= 2445.98 Then
                    desiredZoomLevel = 6
                ElseIf desiredZoom > 2445.98 AndAlso desiredZoom <= 4891.97 Then
                    desiredZoomLevel = 5
                ElseIf desiredZoom > 4891.97 AndAlso desiredZoom <= 9783.94 Then
                    desiredZoomLevel = 4
                ElseIf desiredZoom > 9783.94 AndAlso desiredZoom <= 19567.88 Then
                    desiredZoomLevel = 3
                ElseIf desiredZoom > 19567.88 AndAlso desiredZoom <= 39135.76 Then
                    desiredZoomLevel = 2
                ElseIf desiredZoom > 39135.76 AndAlso desiredZoom <= 78271.52 Then
                    desiredZoomLevel = 1
                Else
                    desiredZoomLevel = 10
                End If

                Me.BingMapControl.ZoomLevel = desiredZoomLevel
            End If
        End If
    End Sub

#End Region


    Private Sub NavigationToAddress(ByVal address As String)
        ' Remove all push pins on the map
        Me.BingMapControl.Children.Clear()

        ' If an address and an application key are specified, use the geocode service to calculate the coordinates
        If Not [String].IsNullOrWhiteSpace(Me.Address) AndAlso Not [String].IsNullOrWhiteSpace(Me.ApplicationKey) Then
            Dim request As New GeocodeRequest()
            request.Credentials = New Credentials() With { _
             .ApplicationId = Me.ApplicationKey _
            }
            request.Query = Me.Address

            ' Name of binding is from ServiceReferences.ClientConfig
            Dim binding As New BasicHttpBinding(BasicHttpSecurityMode.None)
            binding.MaxBufferSize = Int32.MaxValue
            binding.MaxReceivedMessageSize = Int32.MaxValue

            Dim clientService As GeocodeServiceClient = New GeocodeServiceClient(binding, BingMap.GeocodeAddress)
            AddHandler clientService.GeocodeCompleted, AddressOf GeocodeServiceClient_GeocodeCompleted
            clientService.GeocodeAsync(request)
        End If
    End Sub

End Class