' ----------------------------------------------------------------------------------
' Microsoft Developer & Platform Evangelism
' 
' Copyright (c) Microsoft Corporation. All rights reserved.
' 
' THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, 
' EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES 
' OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.
' ----------------------------------------------------------------------------------
' The example companies, organizations, products, domain names,
' e-mail addresses, logos, people, places, and events depicted
' herein are fictitious.  No association with any real company,
' organization, product, domain name, email address, logo, person,
' places, or events is intended or should be inferred.
' ----------------------------------------------------------------------------------

Imports Microsoft.LightSwitch.Presentation
Imports System.Diagnostics
Imports System.Reflection
Imports System.Windows.Resources
Imports System.IO
Imports System.Windows.Markup
Imports System.ComponentModel.Composition

<Export(GetType(IControlFactory))>
<ControlFactory(BingMapControlModule.BingMapControl.GlobalName)>
Public Class BingMapControlFactory
    Implements IControlFactory

    Private _dataTemplate As DataTemplate
    Private Shared Dictionary As ResourceDictionary

    Shared Sub New()
        ' Get resource
        Dim resourceUri As Uri = GetResourceUri()
        Dim streamInfo As StreamResourceInfo = System.Windows.Application.GetResourceStream(resourceUri)
        Debug.Assert(streamInfo IsNot Nothing, "ResourceDictionary load failed.", "Stream not found: {0}", resourceUri.ToString())

        If streamInfo IsNot Nothing Then
            ' Read resource contents
            Dim contents As String = String.Empty
            Using reader = New StreamReader(streamInfo.Stream)
                contents = reader.ReadToEnd()
            End Using

            ' Ensure contents is not empty
            Debug.Assert(Not String.IsNullOrEmpty(contents), "ResourceDictionary load failed.", "Resource was empty: {0}", resourceUri.ToString())

            ' Load the contents as resource dictionary
            BingMapControlFactory.Dictionary = XamlReader.Load(contents)
            Debug.Assert(BingMapControlFactory.Dictionary IsNot Nothing, "ResourceDictionary load failed.", "Resource wasn't ResourceDictionary: {0}", resourceUri.ToString())
        End If

    End Sub
    Public Sub New(ByVal templateKey As String)
        _dataTemplate = BingMapControlFactory.GetDataTemplate(templateKey)
    End Sub
    Public Sub New()
        MyClass.New("BingMapTemplate")
    End Sub
    ReadOnly Property DataTemplate() As DataTemplate Implements IControlFactory.DataTemplate
        Get
            Return _dataTemplate
        End Get
    End Property

    Function GetDisplayModeDataTemplate(ByVal contentItem As IContentItem) As DataTemplate Implements IControlFactory.GetDisplayModeDataTemplate
        Return Nothing
    End Function
    Private Shared Function GetDataTemplate(ByVal templateKey As String) As DataTemplate
        Debug.Assert(BingMapControlFactory.Dictionary.Contains(templateKey), "DataTemplate not found.", "Looking for resource key '{0}'", templateKey)

        Return BingMapControlFactory.Dictionary(templateKey)
    End Function

    Private Shared Function GetResourceUri() As Uri
        Dim assembly As Assembly = assembly.GetExecutingAssembly()
        Dim thisAssemblyName As AssemblyName = New AssemblyName(assembly.FullName)

        Return New Uri(thisAssemblyName.Name + ";component/Controls/DataTemplates.xaml", UriKind.Relative)
    End Function
End Class